/*
 *  FONT.C  'central processing file' of the font package
 *
 *  stroked fonts:	      (c) 1988 - 1993 Borland International
 *  BGIFONT package:	      (c) 1988 - 1989 Borland International
 *  'package derived from the BGIFONT package from Borland International'
 *  SVGAKIT, VESA extensions: (c) 1993 Johannes Martin
 *  EMX/GCC:		      (c) 1992 - 1993 Eberhard Mattes
 *  functions not covered by the above listed copyrights:
 *			      (c) 1993 Matthias Burian
 *
 *  $Author: M_Burian $
 *  $Date: 1993/08/04 19:54:18 $
 *  $Source: c:/work/bc/chr/RCS/font.c%v $
 *  $Revision: 0.1 $
 */

#include "font.h"
#include <malloc.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <graph.h>
#include <conio.h>

#define HEADERSIZE 0x80

static int font_loaded = -1;	       /* is there a font already loaded ? */
static int _xscale = DEFAULT_WIDTH;    /* desired height (in pixels) */
static int _yscale = DEFAULT_HEIGHT;   /* desired width (in pixels) */
static int cur_direction = NORMAL;     /* current writing direction */
static int offsettable[256], char_width[256];
static unsigned char *offsets[256];
static unsigned char *fontptr, *offsetptr, *tmpptr;
static int _char_height = 0;
static int first_char, n_chars, char_offsets;
static int orgcap, orgbase, orgdec;

/*
 *  G_SET_FONT_DIMENSIONS() sets height and width of the characters
 */
void g_set_font_dimensions(int height, int width)
{
    if((height <= 0) && (height != DEFAULT_HEIGHT))
	return;
    else
	_yscale = height;
    if((width <= 0) && (width != DEFAULT_WIDTH))
	return;
    else
	_xscale = width;
}				       /* end g_set_font_height() */

/*
 *  G_GET_FONT_DIMENSIONS() returns the current scaling factors
 */
void g_get_font_dimensions(int *width, int *height)
{
    *width  = _xscale;
    *height = _yscale;
}				       /* end g_get_font_dimensions() */

/*
 *  G_GET_CURRENT_FONT()  returns the number of the already loaded font
 */
int g_get_current_font(void)
{
    return(font_loaded);
}				       /* end g_get_current_font() */

/*
 *  G_USE_FONT()  installs the desired font to be further used
 */
int g_use_font(unsigned int font)
{
    char buf[512], *env;
    FILE *fontfile;
    long endoffile;
    int i;

    buf[0] = 0x00;

    if((font_loaded != -1) && (font == font_loaded))
	return(font_loaded);

    if(font_loaded != -1)
	free(fontptr);

    if((env = getenv("EMXFONT")) == NULL)
    {
	switch(font)
	{
	    case(TRIPLEX): strcat(buf, "trip.chr"); break;
	    case(SANSSER): strcat(buf, "sans.chr"); break;
	    case(EUROFON): strcat(buf, "euro.chr"); break;
	    case(GOTHICF): strcat(buf, "goth.chr"); break;
	    case(SCRIPTF): strcat(buf, "scri.chr"); break;
	    case(LCOMFON): strcat(buf, "lcom.chr"); break;
	    case(LITTLEF): strcat(buf, "litt.chr"); break;
	    case(TSCRFON): strcat(buf, "tscr.chr"); break;
	    case(SIMPLEX): strcat(buf, "simp.chr"); break;
	    default:	   strcat(buf, "simp.chr"); break;
	}

	if((fontfile = fopen(buf, "rb")) == NULL)
	    return(-1);
	else
	    goto len_label;
    }

    strcpy(buf, env);
    strcat(buf, "/");

    switch(font)
    {
	case(TRIPLEX): strcat(buf, "trip.chr"); break;
	case(SANSSER): strcat(buf, "sans.chr"); break;
	case(EUROFON): strcat(buf, "euro.chr"); break;
	case(GOTHICF): strcat(buf, "goth.chr"); break;
	case(SCRIPTF): strcat(buf, "scri.chr"); break;
	case(LCOMFON): strcat(buf, "lcom.chr"); break;
	case(LITTLEF): strcat(buf, "litt.chr"); break;
	case(TSCRFON): strcat(buf, "tscr.chr"); break;
	case(SIMPLEX): strcat(buf, "simp.chr"); break;
	default:       strcat(buf, "simp.chr"); break;
    }

    if((fontfile = fopen(buf, "rb")) == NULL)
	return(-1);

len_label:
    fseek(fontfile, 0L, SEEK_END);
    endoffile = ftell(fontfile);
    rewind(fontfile);

    if((fontptr = (unsigned char*)malloc(endoffile*sizeof(char))) == NULL)
    {
	fclose(fontfile);
	return(-1);
    }

    if(fread(fontptr,sizeof(unsigned char),endoffile,fontfile) != endoffile)
    {
	fclose(fontfile);
	free(fontptr);
	return(-1);
    }

    for(i = 0; i < 256; i++)
    {
	char_width[i] = 0;
	offsettable[i] = 0;
	offsets[i] = NULL;
    }

    fclose(fontfile);

    tmpptr = fontptr + 0x80;

    first_char = (int) *(tmpptr+4);
    n_chars = (int) *(tmpptr+1);
    char_offsets = 256 * (int) *(tmpptr+6) * (int) *(tmpptr+5);
    orgcap = (int) *(tmpptr+8);
    orgbase = (int) *(tmpptr+9);
    orgdec = (int) *(tmpptr+10);
    orgdec -= 256;
    offsetptr = tmpptr + 0x0f;

    for(i = 0; i < n_chars; i++)
	offsettable[(int)first_char + i] =
	    *(offsetptr+2*(i+1))*256 + *(offsetptr+2*(i+1)-1);

    tmpptr = (unsigned char *)(offsetptr + n_chars*2*sizeof(char) + 1);

    for(i = 0; i < n_chars; i++)
	char_width[(int)first_char + i] = *(tmpptr+i);

    tmpptr += sizeof(char) * n_chars;

    for(i = 0; i < n_chars; i++)
	offsets[(int)first_char + i] = tmpptr +
	    offsettable[(int)first_char + i];

    _char_height = orgcap - orgdec;

    font_loaded = font;
    return(font_loaded);
}				       /* end g_use_font() */

/*
 *  G_SET_TEXT_DIRECTION() sets the appropriate text drawing direction
 */
void g_set_text_direction(int direction)
{
    switch(direction)
    {
	case(UP):	cur_direction = UP;	break;
	case(DOWN):	cur_direction = DOWN;	break;
	default:	cur_direction = NORMAL; break;
    }
}				       /* end g_set_text_direction() */

/*
 *  G_GET_TEXT_DIRECTION() returns the current setting for drawing direction
 */
int g_get_text_direction(void)
{
    return(cur_direction);
}

/*
 *  G_GET_STRING_WIDTH() returns the current string width in pixels
 */
int g_get_string_width(char *str)
{
    int  sum = 0;
    char *tmpstr;

    tmpstr = str;

    if(_xscale == DEFAULT_WIDTH)
    {
	while(*tmpstr != 0x00)
	{
	    sum += char_width[(int)(*tmpstr)];
	    tmpstr++;
	}
    }
    else
    {
	while(*tmpstr != 0x00)
	{
	    sum += _xscale;
	    tmpstr++;
	}
    }
    return(sum);
}				       /* end g_get_string_width() */

/*
 *  G_DRAW_CHAR()  draws a character at x/y with color 'color'
 */
static void g_draw_char(int x, int y, char ch, int color)
{
    int  x1 = 0, y1 = 0, x2, y2, flag1 = 0, flag2 = 0, flag, width, height;
    unsigned char xmove, ymove;

    width = char_width[(int)ch];
    height = _char_height;

    tmpptr = offsets[(int)ch];

    do
    {
	flag1 = 0;
	flag2 = 0;

	xmove = *tmpptr;
	ymove = *(tmpptr+1);
	tmpptr += 2 /* *sizeof(char) */ ;

	x2 = (int)xmove;
	y2 = (int)ymove;

	if(x2 >= 128)
	{
	    flag1 = 1;
	    x2 -= 128;
	}
	if(x2 >= 64)
	    x2 -= 128;

	if(y2 >= 128)
	{
	    flag2 = 1;
	    y2 -= 128;
	}
	if(y2 >= 64)
	    y2 -= 128;

	flag = flag1 + flag2;

	if(_xscale != DEFAULT_WIDTH)
	    x2 = _xscale*x2 / width;
	if(_yscale != DEFAULT_HEIGHT)
	    y2 = _yscale*y2 / height;

	if(flag == 0)
	    break;

	switch(flag)
	{
	    case(2): if((cur_direction == NORMAL) || (cur_direction == RIGHT))
			 g_line(x+x1, y-y1, x+x2, y-y2, color);
		     else
		     if(cur_direction == UP)
			 g_line(x-y1, y-x1, x-y2, y-x2, color);
		     else
		     if(cur_direction == DOWN)
			 g_line(x+y1, y+x1, x+y2, y+x2, color);
		     break;

	    case(1): break;
	    default: break;
	}

	x1 = x2;
	y1 = y2;
    }
    while(1);
}				       /* end g_draw_char() */

/*
 *  G_DRAW_STRING()  draws a string at x/y with color 'color'
 */
void g_draw_string(int x, int y, char *str, int color)
{
    char tmpchar;
    int offset;

    if(font_loaded == -1)	       /* no fontfile loaded */
	return;

    while(*str != 0x00)
    {
	tmpchar = *str;
	g_draw_char(x, y, tmpchar, color);

	if(_xscale == DEFAULT_WIDTH)
	    offset = char_width[(int)tmpchar];
	else
	    offset = _xscale;

	switch(cur_direction)
	{
	    case(UP):	y -= offset; break;
	    case(DOWN): y += offset; break;
	    default:	x += offset; break;
	}

	str++;
    }
}				       /* end g_draw_string() */

/*
 *  end FONT.C
 */
