#include <stdio.h>
#include <stdlib.h>
#include <getopt.h>
#include "graph.h"
#include "vesa.h"

#pragma pack(1)

FILE *Open_Targa(char *name, int *width, int *height)
{
  FILE *f;
  int  i;

  if ((f = fopen(name, "rb")) == NULL)
    return(NULL);
  for (i = 0; i < 12; i++)
    fgetc(f);
  *width  = fgetc(f);
  *width  |= fgetc(f) << 8;
  *height = fgetc(f);
  *height |= fgetc(f) << 8;
  return(f);
}

void Close_Targa(FILE *f)
{
  fclose(f);
}

int Read_Targa(FILE *f, char *pixel, int n)
{
  n *= 3;
  while (n-- > 0)
    *pixel++ = fgetc(f);
  return(ferror(f) || feof(f));
}

int Show_Targa(char *filename)
{
  FILE *f;
  int width, height, x, y;
  char pixel[3];

  if ((f = Open_Targa(filename, &width, &height)) == NULL)
    return(1);
  for (y = 0; y < height; y++)
    for (x = 0; x < width; x++)
      {
        if (Read_Targa(f, pixel, 1))
          return(1);
        switch (g_colors)
          {
            case 32768:
              g_set(x, y, RGB15(pixel[1], pixel[0], pixel[2]));
              break;
            case 65536:
              g_set(x, y, RGB16(pixel[1], pixel[0], pixel[2]));
              break;
            default:
              g_set(x, y, RGB24(pixel[1], pixel[0], pixel[2]));
          }
      }
  return(0);
}

static void usage (void)
{
  VESAINFO     vinfo;
  VESAMODEINFO minfo;
  PVESAWORD    pmode;

  fprintf(stderr, "Usage: tgaview [-f<name>] [-m#]\n\n");
  fprintf(stderr, "-f<name>  View file name [test.tga]\n");
  fprintf(stderr, "-m#       Select video mode # [%x]\n", G_MODE_VGA_L);
  if (VesaGetInfo(&vinfo))
    {
      fprintf(stderr, "       Available modes:\n");
      fprintf(stderr, "       ModeNr    xres yres colors\n");
      pmode = vinfo.Modes;
      while (*pmode != 0xFFFF)
        {
          if (VesaGetModeInfo(*pmode, &minfo))
            {
              if ((minfo.ModeAttributes & 0x10) == 0x10)
                {
                  int colors;

                  if (minfo.RedMaskSize != 0)
                    colors = 1 << (minfo.RedMaskSize + minfo.GreenMaskSize +
                                   minfo.BlueMaskSize);
                  else
                    colors = 1 << minfo.NumberOfBitsPerPixel;
                  fprintf(stderr, "%13x%8i%5i %i\n", *pmode, minfo.Width,
                                  minfo.Height, colors);
                }
            }
          else
            fprintf(stderr, "%13x --- VesaGetModeInfo failed ---\n", *pmode);
          pmode++;
        }
      fprintf(stderr, "\n");
    }
  exit (1);
}

int main (int argc, char *argv[])
{
  int rc, video_mode;
  char *filename, c;

  video_mode = G_MODE_VGA_L;
  filename   = "test.tga";
  while ((c = getopt (argc, argv, "f:m:")) != EOF)
    switch (c)
      {
      case 'm':
        if ((sscanf(optarg, "%x", &video_mode) != 1) || (video_mode < 0))
          usage ();
        break;
      case 'f':
        filename = optarg;
        break;
      default:
        usage ();
      }
  if (optind < argc)
    usage ();
  if (!g_mode (video_mode))
    {
      fprintf (stderr, "Cannot switch to graphics mode %i\n", video_mode);
      return (1);
    }
  if ((rc = Show_Targa(filename)) != 0)
    fputc(0x07, stdout);
  getchar();
  g_mode (G_MODE_OFF);
  if (rc)
    {
      fprintf (stderr, "File not found: %s\n", filename);
      return (1);
    }
  return (0);
}
