/* VESA package for emx/gcc --- Copyright (c) 1993 by Johannes Martin */
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <sys/hw.h>

#define INCL_WIN
#define INCL_DOSQUEUES
#define INCL_DOSPROCESS
#define INCL_DOSFILEMGR
#define INCL_DOSSEMAPHORES
#define INCL_VIO
#define INCL_AVIO

#define PAL_COUNT 256
#define TEXT_COLS 80
#define TEXT_ROWS 25

#include <os2emx.h>
#include <os2thunk.h>

#include "vesa_pm.h"
#include "pmscreen.h"

static DATABUFFER *DataBuffer;
static HPIPE      hpipe1, hpipe2, hpipe3, hpipe4;
static BOOL       UsePalette;
static BOOL       UseFullPalette;
static BOOL       OriginalSize;
static BOOL       HasPalMan;
static USHORT     Intervall;

MRESULT SettingsDlgProc(HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2)
{
  switch (msg)
    {
      case WM_INITDLG:
        {
          WinEnableWindow(WinWindowFromID(hwnd, DID_USEPAL), HasPalMan);
          WinEnableWindow(WinWindowFromID(hwnd, DID_FULLPAL), HasPalMan && UsePalette);
          WinSetDlgItemShort(hwnd, DID_INTERVALL, Intervall, FALSE);
          WinCheckButton(hwnd, DID_USEPAL, UsePalette);
          WinCheckButton(hwnd, DID_FULLPAL, UseFullPalette);
          WinCheckButton(hwnd, DID_ORGSIZE, OriginalSize);
          return(0);
        }
      case WM_CONTROL:
        switch(SHORT2FROMMP(mp1))
          {
            case BN_CLICKED:
              if (SHORT1FROMMP(mp1) == DID_USEPAL)
                {
                  ULONG State = !WinQueryButtonCheckstate(hwnd, DID_USEPAL);
                  WinCheckButton(hwnd, DID_USEPAL, State);
                  WinEnableWindow(WinWindowFromID(hwnd, DID_FULLPAL), State);
                }
              break;
          }
        break;
      case WM_COMMAND:
        switch (COMMANDMSG(&msg)->cmd)
          {
            case DID_OK:
              {
                WinQueryDlgItemShort(hwnd, DID_INTERVALL, &Intervall, FALSE);
                UsePalette     = WinQueryButtonCheckstate(hwnd, DID_USEPAL);
                UseFullPalette = WinQueryButtonCheckstate(hwnd, DID_FULLPAL);
                OriginalSize   = WinQueryButtonCheckstate(hwnd, DID_ORGSIZE);
                WinDismissDlg(hwnd, TRUE);
                return(0);
              }
            case DID_CANCEL:
              WinDismissDlg(hwnd, FALSE);
              return(0);
          }
        break;
    }
  return(WinDefDlgProc(hwnd, msg, mp1, mp2));
}

void WriteChar(CHAR chChar, CHAR chScan)
{
  ULONG written;

  if (chChar == 0)
    {
      DosWrite(hpipe2, &chChar, sizeof(chChar), &written);
      DosWrite(hpipe2, &chScan, sizeof(chScan), &written);
    }
  else
    {
      DosWrite(hpipe1, &chChar, sizeof(chChar), &written);
      DosWrite(hpipe2, &chChar, sizeof(chChar), &written);
    }
}

void InstallMenu(HWND hwnd)
{
  CHAR *szMenuText[3] = { NULL, "~Update Screen", "~PM-Screen Settings" };
  MENUITEM mi[3]      = {{ MIT_END, MIS_SEPARATOR, 0, 0, (HWND) NULL, (ULONG) NULL },
                         { MIT_END, MIS_TEXT, 0, IDM_UPDATE, (HWND) NULL, (ULONG) NULL },
                         { MIT_END, MIS_TEXT, 0, IDM_SETTINGS, (HWND) NULL, (ULONG) NULL }};
  HWND     hwndSysMenu, hwndSysSubMenu;
  MENUITEM miSysMenu;
  SHORT    sItem, idSysMenu;

  hwndSysMenu = WinWindowFromID(WinQueryWindow(hwnd, QW_PARENT), FID_SYSMENU);
  idSysMenu   = SHORT1FROMMR(WinSendMsg(hwndSysMenu, MM_ITEMIDFROMPOSITION, NULL, NULL));
  WinSendMsg(hwndSysMenu, MM_QUERYITEM,  MPFROM2SHORT(idSysMenu, FALSE), MPFROMP(&miSysMenu));
  hwndSysSubMenu = miSysMenu.hwndSubMenu;
  for (sItem = 0; sItem < 3; sItem++)
     WinSendMsg(hwndSysSubMenu, MM_INSERTITEM, MPFROMP(mi+sItem), MPFROMP(szMenuText[sItem]));
}

BOOL TestPalMan(HDC hdc)
{
  ULONG caps;

  DevQueryCaps(hdc, CAPS_ADDITIONAL_GRAPHICS, 1, &caps);
  return((caps & CAPS_PALETTE_MANAGER) != 0);
}

VOID MakePalette(HWND hwnd, HPS hps, HPAL *hPal)
{
  ULONG i, color;
  ULONG alTable[PAL_COUNT];

  for (i = 0; i < PAL_COUNT; i++)
    alTable[i] = PC_RESERVED * 16777216
               + DataBuffer->bmi.argbColor[i].bRed * 65536
               + DataBuffer->bmi.argbColor[i].bGreen * 256
               + DataBuffer->bmi.argbColor[i].bBlue;

  *hPal = GpiCreatePalette(WinQueryAnchorBlock(hwnd),
                           LCOL_PURECOLOR | (UseFullPalette ? LCOL_OVERRIDE_DEFAULT_COLORS : 0),
                           LCOLF_CONSECRGB, PAL_COUNT, alTable);
  GpiSelectPalette(hps, *hPal);
  WinRealizePalette(hwnd, hps, &color);
}

VOID RemovePalette(HPS hps, HPAL *hPal)
{
  GpiSelectPalette(hps, 0);
  GpiDeletePalette(*hPal);
  *hPal = 0;
}

MRESULT GraphWndProc(HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2)
{
  static BOOL do_update;
  static HMTX hmtxPS;
  static HPS  hpsBuffer;
  static HPAL hPal;
  static HDC  hdc;

  switch (msg)
    {
      case WM_CREATE:
        {
          SIZEL sizl = { 0L, 0L };
          HAB   hab;

          DosCreateMutexSem(NULL, &hmtxPS, 0, FALSE);
          hab       = WinQueryAnchorBlock(hwnd);
          hdc       = WinOpenWindowDC(hwnd);
          hpsBuffer = GpiCreatePS(hab, hdc, &sizl, PU_PELS | GPIF_DEFAULT |
                                  GPIT_MICRO | GPIA_ASSOC );
          DataBuffer->hwndGraph = hwnd;
          DataBuffer->modified  = TRUE;
          HasPalMan             = TestPalMan(hdc);
          UsePalette            = FALSE;
          UseFullPalette        = FALSE;
          OriginalSize          = FALSE;
          Intervall             = 3000;
          do_update             = TRUE;
          hPal                  = 0;
          InstallMenu(hwnd);
          WinStartTimer(hab, hwnd, ID_TIMER, Intervall);
          break;
        }
      case WM_CHAR:
        {
          if (CHARMSG(&msg)->fs & KC_KEYUP)
            return(0);
          if (CHARMSG(&msg)->fs & KC_CHAR)
            {
              WriteChar(CHARMSG(&msg)->chr, 0);
              return(0);
            }
          if (CHARMSG(&msg)->fs & KC_VIRTUALKEY)
            {
              switch (CHARMSG(&msg)->vkey)
                {
                  case VK_ESC:
                    WriteChar(0x1b, 0);
                    break;
                }
              return(0);
            }
          if ((CHARMSG(&msg)->fs & KC_CTRL) && (CHARMSG(&msg)->chr != 0))
            {
              WriteChar(CHARMSG(&msg)->chr & 0x1f, 0);
              return(0);
            }
          return(0);
        }
      case WM_COMMAND:
        switch (COMMANDMSG(&msg) -> cmd)
          {
            case IDM_SETTINGS:
              {
                static BOOL inDlg = 0;
                ULONG rc;

                if (inDlg)
                  return(0);
                inDlg = 1;
                rc = WinDlgBox(HWND_DESKTOP, hwnd, SettingsDlgProc, 0, ID_SETTINGS, NULL);
                inDlg = 0;
                if (rc)
                  {
                    SWP swp;

                    WinStopTimer(WinQueryAnchorBlock(hwnd), hwnd, ID_TIMER);
                    WinStartTimer(WinQueryAnchorBlock(hwnd), hwnd, ID_TIMER, Intervall);
                    RemovePalette(hpsBuffer, &hPal);

                    WinQueryWindowPos(WinQueryWindow(hwnd, QW_PARENT), &swp);
                    if (((swp.fl & SWP_MINIMIZE) == 0) && OriginalSize &&
                        ((DataBuffer->pointl[1].y - DataBuffer->pointl[0].y != DataBuffer->pointl[3].y) ||
                         (DataBuffer->pointl[1].x - DataBuffer->pointl[0].x != DataBuffer->pointl[3].x)))
                      {
                        RECTL rectl;

                        rectl.xLeft   = 0;
                        rectl.yBottom = 0;
                        rectl.xRight  = DataBuffer->pointl[3].x;
                        rectl.yTop    = DataBuffer->pointl[3].y;
                        WinCalcFrameRect(WinQueryWindow(hwnd, QW_PARENT), &rectl, FALSE);
                        WinSetWindowPos(WinQueryWindow(hwnd, QW_PARENT), 0, 0, 0,
                                        rectl.xRight, rectl.yTop, SWP_SIZE);
                      }
                  }
                else
                  return(0);
              }
            case IDM_UPDATE:
              WinInvalidateRect(hwnd, NULL, FALSE);
              return(0);
          }
        break;
      case WM_DONTUPDATE:
        do_update = FALSE;
        DosPostEventSem(DataBuffer->hev);
        return(0);
      case WM_NEWMODE:
        DataBuffer->bmi.cbFix           = 64;
        DataBuffer->bmi.cx              = DataBuffer->pointl[3].x;
        DataBuffer->bmi.cy              = DataBuffer->pointl[3].y;
        DataBuffer->bmi.cPlanes         = 1;
        DataBuffer->bmi.cBitCount       = 8;
        DataBuffer->bmi.ulCompression   = BCA_UNCOMP;
        DataBuffer->bmi.cbImage         = 0;
        DataBuffer->bmi.cxResolution    = 2000;
        DataBuffer->bmi.cyResolution    = 2000;
        DataBuffer->bmi.cclrUsed        = 256;
        DataBuffer->bmi.cclrImportant   = 256;
        DataBuffer->bmi.usUnits         = BRU_METRIC;
        DataBuffer->bmi.usReserved      = 0;
        DataBuffer->bmi.usRecording     = BRA_BOTTOMUP;
        DataBuffer->bmi.usRendering     = BRH_NOTHALFTONED;
        DataBuffer->bmi.cSize1          = 0;
        DataBuffer->bmi.cSize2          = 0;
        DataBuffer->bmi.ulColorEncoding = BCE_RGB;
        DataBuffer->bmi.ulIdentifier    = 0;
        if (DataBuffer->GraphBuffer != 0)
          {
            DosGetSharedMem(DataBuffer->GraphBuffer, PAG_READ);
            WinSetWindowPos(WinQueryWindow(DataBuffer->hwndText, QW_PARENT),
                            0, 0, 0, 0, 0, SWP_SHOW | SWP_MINIMIZE);
            WinSetWindowPos(WinQueryWindow(DataBuffer->hwndGraph, QW_PARENT),
                            0, 0, 0, 0, 0, SWP_SHOW | SWP_RESTORE);
          }
        else
          {
            WinSetWindowPos(WinQueryWindow(DataBuffer->hwndGraph, QW_PARENT),
                            0, 0, 0, 0, 0, SWP_SHOW | SWP_MINIMIZE);
            WinSetWindowPos(WinQueryWindow(DataBuffer->hwndText, QW_PARENT),
                            0, 0, 0, 0, 0, SWP_SHOW | SWP_RESTORE);
          }
      case WM_UPDATE:
        do_update = TRUE;
        DataBuffer->modified = TRUE;
        WinInvalidateRect(hwnd, NULL, FALSE);
        return(0);
      case WM_CHGPALETTE:
        if (hPal)
          {
            ULONG i;
            ULONG alTable[PAL_COUNT];

            for (i = 0; i < PAL_COUNT; i++)
              alTable[i] = PC_RESERVED * 16777216
                         + DataBuffer->bmi.argbColor[i].bRed * 65536
                         + DataBuffer->bmi.argbColor[i].bGreen * 256
                         + DataBuffer->bmi.argbColor[i].bBlue;
            WinRequestMutexSem(hmtxPS, SEM_INDEFINITE_WAIT);
            GpiAnimatePalette(hPal, LCOLF_CONSECRGB, 0, PAL_COUNT, alTable);
            DosReleaseMutexSem(hmtxPS);
          }
        else
          DataBuffer->modified = TRUE;
        DosPostEventSem(DataBuffer->hev);
        return(0);
      case WM_SIZE:
        {
          SWP swp;

          WinQueryWindowPos(WinQueryWindow(hwnd, QW_PARENT), &swp);
          WinQueryWindowRect(hwnd, (PRECTL) DataBuffer->pointl);
          if (((swp.fl & SWP_MINIMIZE) == 0) && OriginalSize &&
              ((DataBuffer->pointl[1].y - DataBuffer->pointl[0].y != DataBuffer->pointl[3].y) ||
               (DataBuffer->pointl[1].x - DataBuffer->pointl[0].x != DataBuffer->pointl[3].x)))
            {
              RECTL rectl;

              rectl.xLeft   = 0;
              rectl.yBottom = 0;
              rectl.xRight  = DataBuffer->pointl[3].x;
              rectl.yTop    = DataBuffer->pointl[3].y;
              WinCalcFrameRect(WinQueryWindow(hwnd, QW_PARENT), &rectl, FALSE);
              WinSetWindowPos(WinQueryWindow(hwnd, QW_PARENT), 0, 0, 0,
                              rectl.xRight, rectl.yTop, SWP_SIZE);
            }
          else
            WinInvalidateRect(hwnd, NULL, FALSE);
        }
        break;
      case WM_TIMER:
        if (do_update && DataBuffer->modified)
          WinInvalidateRect(hwnd, NULL, FALSE);
        break;
      case WM_PAINT:
        {
          HPS hps;

          WinRequestMutexSem(hmtxPS, SEM_INDEFINITE_WAIT);
          if (UsePalette && !hPal)
            MakePalette(hwnd, hpsBuffer, &hPal);
          else
            if (hPal && !UsePalette)
              RemovePalette(hpsBuffer, &hPal);
          hps = WinBeginPaint(hwnd, hpsBuffer, NULL);
          if (do_update && (DataBuffer->GraphBuffer != NULL))
            {
              GpiDrawBits(hps, DataBuffer->GraphBuffer, (PVOID) &DataBuffer->bmi,
                          4, DataBuffer->pointl, ROP_SRCCOPY, BBO_IGNORE);
              DataBuffer->modified = FALSE;
            }
          else
            GpiErase(hps);
          WinEndPaint(hps);
          DosReleaseMutexSem(hmtxPS);
        }
        return(0);
      case WM_REALIZEPALETTE:
        if (hPal)
          {
            ULONG cclr;

            WinRequestMutexSem(hmtxPS, SEM_INDEFINITE_WAIT);
            WinRealizePalette(hwnd, hpsBuffer, &cclr);
            DosReleaseMutexSem(hmtxPS);
          }
        break;
      case WM_FINISH:
        WinPostMsg(WinQueryWindow(hwnd, QW_PARENT), WM_QUIT, NULL, NULL);
        return(0);
      case WM_CLOSE:
        if (WinMessageBox(HWND_DESKTOP, hwnd, "Are you sure?", "Close Window?",
                          0, MB_YESNO | MB_ICONQUESTION) == MBID_YES)
          WinPostMsg(WinQueryWindow(hwnd, QW_PARENT), WM_QUIT, NULL, NULL);
        return(0);
      case WM_DESTROY:
        WinStopTimer(WinQueryAnchorBlock(hwnd), hwnd, ID_TIMER);
        if (hPal)
          RemovePalette(hpsBuffer, &hPal);
        if (hpsBuffer)
          WinReleasePS(hpsBuffer);
        break;
    }
  return(WinDefWindowProc(hwnd, msg, mp1, mp2));
}

void StdOutThread(ULONG x)
{
  ULONG read;
  CHAR  c;

  while (!DosRead(hpipe3, &c, sizeof(c), &read))
    WinPostMsg(DataBuffer->hwndText, WM_NEWCHAR, MPFROMLONG(c), NULL);
}

void StdErrThread(ULONG x)
{
  ULONG read;
  CHAR  c;

  while (!DosRead(hpipe4, &c, sizeof(c), &read))
    WinPostMsg(DataBuffer->hwndText, WM_NEWCHAR, MPFROMLONG(c), NULL);
}

#ifdef 0
MRESULT TextWndProc(HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2)
{
  static CHAR  TextBuffer[TEXT_ROWS][TEXT_COLS];
  static ULONG TextHeight, WindowHeight, TextX, TextY;

  switch (msg)
    {
      case WM_CREATE:
        {
          FONTMETRICS fm;
          HPS         hps;

          DataBuffer->hwndText = hwnd;
          InstallMenu(hwnd);
          hps = WinGetPS(hwnd);
          GpiQueryFontMetrics(hps, sizeof(fm), &fm);
          WinReleasePS(hps);
          TextHeight = fm.lMaxBaselineExt;
          TextX      = 0;
          TextY      = 0;
          memset(TextBuffer, 0, sizeof(TextBuffer));
        }
        break;
      case WM_SIZE:
        WindowHeight = SHORT2FROMMP(mp2);
        break;
      case WM_NEWCHAR:
        switch (LONGFROMMP(mp1))
          {
            case '\b':
              if (TextX > 0)
                TextX--;
              break;
            case '\r':
              TextX = 0;
              break;
            default:
              TextBuffer[TextY][TextX] = LONGFROMMP(mp1);
              if (TextX < TEXT_COLS - 1)
                {
                  HPS hps;

                  hps = WinGetPS(hwnd);

                  TextX++;
                  break;
                }
              TextX = 0;
            case '\n':
              if (TextY < TEXT_ROWS - 1)
                TextY++;
              else
                {
                  memcpy(TextBuffer[0], TextBuffer[1],
                         (TEXT_ROWS - 1) * sizeof(TextBuffer[0]));
                  memset(TextBuffer[TEXT_ROWS - 1], 0, sizeof(TextBuffer[0]));
                  WinInvalidateRect(hwnd, NULL, FALSE);
                }
              break;
          }
        return(0);
      case WM_CHAR:
        WinSendMsg(DataBuffer->hwndGraph, msg, mp1, mp2);
        break;
      case WM_COMMAND:
        switch (COMMANDMSG(&msg) -> cmd)
          {
            case IDM_SETTINGS:
              WinSendMsg(DataBuffer->hwndGraph, msg, mp1, mp2);
              break;
            case IDM_UPDATE:
              WinInvalidateRect(hwnd, NULL, FALSE);
              return(0);
          }
        break;
      case WM_PAINT:
        {
          HPS    hps;
          ULONG  i;
          POINTL pointl;

          hps = WinBeginPaint(hwnd, NULL, NULL);
          pointl.x = 0;
          pointl.y = WindowHeight;
          GpiErase(hps);
          for (i = 0; i < 25; i++)
            {
              pointl.y -= TextHeight;
              GpiCharStringAt(hps, &pointl, 80, TextBuffer[i]);
            }
          WinEndPaint(hps);
        }
        return(0);
      case WM_CLOSE:
        if (WinMessageBox(HWND_DESKTOP, hwnd, "Are you sure?", "Close Window?",
                          0, MB_YESNO | MB_ICONQUESTION) == MBID_YES)
          WinPostMsg(WinQueryWindow(hwnd, QW_PARENT), WM_QUIT, NULL, NULL);
        return(0);
    }
  return(WinDefWindowProc(hwnd, msg, mp1, mp2));
}
#endif

MRESULT TextWndProc(HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2)
{
  static HPS  hps;
  static HVPS hvps;

  switch (msg)
    {
      case WM_CREATE:
        {
          HDC   hdc;
          SIZEL sizel;

          DataBuffer->hwndText = hwnd;
          InstallMenu(hwnd);

          hdc = WinOpenWindowDC(hwnd);
          sizel.cx = sizel.cy = 0;
          hps = GpiCreatePS(WinQueryAnchorBlock(hwnd), hdc, &sizel,
                            PU_PELS | GPIF_DEFAULT | GPIT_MICRO | GPIA_ASSOC);
          VioCreatePS(&hvps, TEXT_ROWS, TEXT_COLS, 0, 1, 0);
          VioAssociate(hdc, hvps);
          return(0);
        }
      case WM_SIZE:
        WinDefAVioWindowProc(hwnd, msg, LONGFROMMP(mp1), LONGFROMMP(mp2));
        return(0);
      case WM_NEWCHAR:
        {
          char c = LONGFROMMP(mp1);

          VioWrtTTY(&c, sizeof(c), hvps);
          return(0);
        }
      case WM_CHAR:
        WinSendMsg(DataBuffer->hwndGraph, msg, mp1, mp2);
        break;
      case WM_COMMAND:
        switch (COMMANDMSG(&msg) -> cmd)
          {
            case IDM_SETTINGS:
              WinSendMsg(DataBuffer->hwndGraph, msg, mp1, mp2);
              break;
            case IDM_UPDATE:
              WinInvalidateRect(hwnd, NULL, FALSE);
              return(0);
          }
        break;
      case WM_PAINT:
        WinBeginPaint(hwnd, hps, NULL);
        GpiErase(hps);
        VioShowBuf(0, TEXT_ROWS * TEXT_COLS * 2, hvps);
        WinEndPaint(hps);
        return(0);
      case WM_CLOSE:
        if (WinMessageBox(HWND_DESKTOP, hwnd, "Are you sure?", "Close Window?",
                          0, MB_YESNO | MB_ICONQUESTION) == MBID_YES)
          WinPostMsg(WinQueryWindow(hwnd, QW_PARENT), WM_QUIT, NULL, NULL);
        return(0);
      case WM_DESTROY:
        VioAssociate(0, hvps);
        VioDestroyPS(hvps);
        GpiDestroyPS(hps);
        return(0);
    }
  return(WinDefWindowProc(hwnd, msg, mp1, mp2));
}

int main(int argc, char *argv[])
{
  static CHAR  szGraphClass[] = "PMSCREEN_GRAPH";
  static CHAR  szTextClass[]  = "PMSCREEN_TEXT";
  static ULONG flFrameFlags   = FCF_TITLEBAR      | FCF_SYSMENU   |
                                FCF_SIZEBORDER    | FCF_MINMAX    |
                                FCF_SHELLPOSITION | FCF_TASKLIST  |
                                FCF_ICON;
  HAB   hab;
  HMQ   hmq;
  QMSG  qmsg;
  HWND  hwndGraphFrame, hwndGraphClient;
  HWND  hwndTextFrame, hwndTextClient;
  ULONG action;

  if (argc != 2)
    {
      DataBuffer = malloc(sizeof(DATABUFFER));
      memset(DataBuffer, 0, sizeof(DATABUFFER));
    }
  else
    {
      DataBuffer = (DATABUFFER *) atoi(argv[1]);
      DosGetSharedMem(DataBuffer, PAG_READ | PAG_WRITE);
    }
  hab = WinInitialize(0);
  hmq = WinCreateMsgQueue(hab, 0);
  WinRegisterClass(hab, szGraphClass, GraphWndProc, 0, 0);
  WinRegisterClass(hab, szTextClass, TextWndProc, 0, 0);
  hwndTextFrame  = WinCreateStdWindow(HWND_DESKTOP, WS_VISIBLE, &flFrameFlags,
                                      szTextClass, "PM-Screen Text Window",
                                      0, 0, ID_PMSCREEN, &hwndTextClient);
  hwndGraphFrame = WinCreateStdWindow(HWND_DESKTOP, 0, &flFrameFlags,
                                      szGraphClass, "PM-Screen Graphics Window",
                                      0, 0, ID_PMSCREEN, &hwndGraphClient);
  WinSetWindowPos(hwndGraphFrame, 0, 0, 0, 0, 0, SWP_SHOW | SWP_MINIMIZE);
  if (argc != 2)
    WinMessageBox(HWND_DESKTOP, hwndTextClient, "PMSCREEN is not intended to be called"
                  " from command line.", "Error", 0, MB_OK | MB_ICONEXCLAMATION);
  else
    {
      TID tid;

      DosOpenEventSem(NULL, &DataBuffer->hev);
      DosPostEventSem(DataBuffer->hev);
      DosWaitNPipe(DataBuffer->Pipe1Name, -1);
      DosOpen(DataBuffer->Pipe1Name, &hpipe1, &action, 0, FILE_NORMAL,
              OPEN_ACTION_CREATE_IF_NEW | OPEN_ACTION_OPEN_IF_EXISTS,
              OPEN_SHARE_DENYNONE | OPEN_ACCESS_WRITEONLY, NULL);
      DosSetNPHState(hpipe1, NP_NOWAIT);
      DosWaitNPipe(DataBuffer->Pipe2Name, -1);
      DosOpen(DataBuffer->Pipe2Name, &hpipe2, &action, 0, FILE_NORMAL,
              OPEN_ACTION_CREATE_IF_NEW | OPEN_ACTION_OPEN_IF_EXISTS,
              OPEN_SHARE_DENYNONE | OPEN_ACCESS_WRITEONLY, NULL);
      DosSetNPHState(hpipe2, NP_NOWAIT);
      DosWaitNPipe(DataBuffer->Pipe3Name, -1);
      DosOpen(DataBuffer->Pipe3Name, &hpipe3, &action, 0, FILE_NORMAL,
              OPEN_ACTION_CREATE_IF_NEW | OPEN_ACTION_OPEN_IF_EXISTS,
              OPEN_SHARE_DENYNONE | OPEN_ACCESS_READONLY, NULL);
      DosWaitNPipe(DataBuffer->Pipe4Name, -1);
      DosOpen(DataBuffer->Pipe4Name, &hpipe4, &action, 0, FILE_NORMAL,
              OPEN_ACTION_CREATE_IF_NEW | OPEN_ACTION_OPEN_IF_EXISTS,
              OPEN_SHARE_DENYNONE | OPEN_ACCESS_READONLY, NULL);
      DosCreateThread(&tid, StdOutThread, 0, 0, 32768);
      DosCreateThread(&tid, StdErrThread, 0, 0, 32768);
      while (WinGetMsg(hab, &qmsg, 0, 0, 0))
        WinDispatchMsg(hab, &qmsg);
      DosPostEventSem(DataBuffer->hev);
      DosCloseEventSem(DataBuffer->hev);
      DosClose(hpipe1);
      DosClose(hpipe2);
      DosClose(hpipe3);
      DosClose(hpipe4);
    }
  WinDestroyWindow(hwndGraphFrame);
  WinDestroyWindow(hwndTextFrame);
  WinDestroyMsgQueue(hmq);
  WinTerminate(hab);
  return(argc != 2);
}
